# =====================================================================================================
from struct import * 
from array import *
from glob import *
import os
import sys
# =====================================================================================================

# =====================================================================================================
# Get base file name with and without extension
def fnoext(fname):
	fnameb = os.path.basename(fname)
	idot  = -fname[::-1].find('.')-1
	if idot:
		fnamex = fnameb[:idot]
	else:
		fnamex = fnameb
	return (fnameb,fnamex)
# =====================================================================================================

# =====================================================================================================
# Create dsf using DTPK file
def dsfdtpk(nout,ndrv,ndtpk):
	# Read in driver file
	dsfbin = array('B', '\x00'*0x10000)
	ddrv = array('B', open(ndrv, 'rb').read())
	szdrv = len(ddrv)
	dsfbin[0:szdrv] = ddrv
	
	# Read in DTPK file
	ddtpk = array('B', open(ndtpk, 'rb').read())
	dsfbin += ddtpk
	
	# Configure driver for either Dreamcast or Naomi
	if len(dsfbin) > 0x1FE000:
		dsfbin[0x50:0x54] = array('B', pack('<I', 0x200))    # set driver config to Naomi if > than 2 MB needed
		
	# Set up sound map
	bank = 0
	dtpk_id = unpack('<I', ddtpk[4:8])[0]
	dsfbin[0x60+bank*4:0x64+bank*4] = array('B', pack('<I', dtpk_id))   # Map bank to DTPK id
	
	# Determine bank command
	bank_cmd = 0xA0001100 | bank
	
	# Determine sequence command
	seq_offset = unpack('<I', ddtpk[0x2C:0x30])[0]
	seq_cmd = unpack('<H', ddtpk[seq_offset+6:seq_offset+8])[0] << 16
	
	# Write sound commands
	dsfbin[0x400:0x404] = array('B', pack('>I', bank_cmd))    # Register DTPK bank
	dsfbin[0x404:0x408] = array('B', pack('>I', seq_cmd))     # Play sequence

	# Write output file
	fo = open('temp.bin','wb')
	fo.write('\x00'*4)    # load address
	fo.write(dsfbin)
	fo.close()
	
	# Create DSF
	print 'Creating %s...' % nout
	os.system('bin2psf dsf 18 temp.bin 2>%s' % os.devnull)
	if os.access(nout,os.F_OK):
		os.remove(nout)
	os.rename('temp.dsf', nout)
	
	# Cleanup
	if os.access('temp.bin',os.F_OK):
		os.remove('temp.bin')
# =====================================================================================================

# =====================================================================================================
if __name__ == '__main__':
	argv = sys.argv
	argc = len(argv)
	if argc < 3:
		print 'AM2 DTPK DSF script v0.00 by kingshriek'
		print 'Usage: python %s <DTPK sound driver> <DTPK sound files...>' % argv[0]
		sys.exit()
	ndrv = argv[1]
	for arg in argv[2:]:
		ndtpks = glob(arg)
		for ndtpk in ndtpks:
			(bdtpk, xdtpk) = fnoext(ndtpk)
			nout = xdtpk + '.dsf'
			dsfdtpk(nout,ndrv,ndtpk)
# =====================================================================================================
	